@@ -12,25 +12,33 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Any, Callable, Optional, Type, Union, cast
+
 import numpy as np
 import tensorflow as tf
 
 from .. import kernels
 from .. import mean_functions as mfn
+from ..base import TensorType
 from ..covariances import Kuf
 from ..inducing_variables import InducingVariables
 from ..probability_distributions import DiagonalGaussian, Gaussian, MarkovGaussian
 from ..quadrature import mvnquad
 from . import dispatch
-from .expectations import quadrature_expectation
+from .expectations import ExpectationObject, PackedExpectationObject, quadrature_expectation
 
 register = dispatch.quadrature_expectation.register
 
 
-NoneType = type(None)
+NoneType: Type[None] = type(None)
+EllipsisType = Any
 
 
-def get_eval_func(obj, inducing_variable, slice=None):
+def get_eval_func(
+    obj: ExpectationObject,
+    inducing_variable: Optional[InducingVariables],
+    slice: Union[slice, EllipsisType, None] = None,
+) -> Callable[[TensorType], tf.Tensor]:
     """
     Return the function of interest (kernel or mean) for the expectation
     depending on the type of :obj: and whether any inducing are given
@@ -45,9 +53,9 @@ def get_eval_func(obj, inducing_variable, slice=None):
             raise TypeError("If `inducing_variable` is supplied, `obj` must be a kernel.")
         return lambda x: tf.transpose(Kuf(inducing_variable, obj, x))[slice]
     elif isinstance(obj, mfn.MeanFunction):
-        return lambda x: obj(x)[slice]
+        return lambda x: obj(x)[slice]  # type: ignore
     elif isinstance(obj, kernels.Kernel):
-        return lambda x: obj(x, full_cov=False)
+        return lambda x: obj(x, full_cov=False)  # type: ignore
 
     raise NotImplementedError()
 
@@ -59,7 +67,14 @@ def get_eval_func(obj, inducing_variable, slice=None):
     object,
     (InducingVariables, NoneType),
 )
-def _quadrature_expectation(p, obj1, inducing_variable1, obj2, inducing_variable2, nghp=None):
+def _quadrature_expectation_gaussian(
+    p: Union[Gaussian, DiagonalGaussian],
+    obj1: ExpectationObject,
+    inducing_variable1: Optional[InducingVariables],
+    obj2: ExpectationObject,
+    inducing_variable2: Optional[InducingVariables],
+    nghp: Optional[int] = None,
+) -> tf.Tensor:
     """
     General handling of quadrature expectations for Gaussians and DiagonalGaussians
     Fallback method for missing analytic expectations
@@ -77,23 +92,29 @@ def _quadrature_expectation(p, obj1, inducing_variable1, obj2, inducing_variable
     if not isinstance(p, DiagonalGaussian):
         cov = p.cov
     else:
-        iskern1 = isinstance(obj1, kernels.Kernel)
-        iskern2 = isinstance(obj2, kernels.Kernel)
-        if iskern1 and iskern2 and obj1.on_separate_dims(obj2):  # no joint expectations required
-            eKxz1 = quadrature_expectation(p, (obj1, inducing_variable1), nghp=nghp)
-            eKxz2 = quadrature_expectation(p, (obj2, inducing_variable2), nghp=nghp)
+        if (
+            isinstance(obj1, kernels.Kernel)
+            and isinstance(obj2, kernels.Kernel)
+            and obj1.on_separate_dims(obj2)
+        ):  # no joint expectations required
+            eKxz1 = quadrature_expectation(
+                p, cast(PackedExpectationObject, (obj1, inducing_variable1)), nghp=nghp
+            )
+            eKxz2 = quadrature_expectation(
+                p, cast(PackedExpectationObject, (obj2, inducing_variable2)), nghp=nghp
+            )
             return eKxz1[:, :, None] * eKxz2[:, None, :]
         cov = tf.linalg.diag(p.cov)
 
     if obj2 is None:
 
-        def eval_func(x):
+        def eval_func(x: TensorType) -> tf.Tensor:
             fn = get_eval_func(obj1, inducing_variable1)
             return fn(x)
 
     else:
 
-        def eval_func(x):
+        def eval_func(x: TensorType) -> tf.Tensor:
             fn1 = get_eval_func(obj1, inducing_variable1, np.s_[:, :, None])
             fn2 = get_eval_func(obj2, inducing_variable2, np.s_[:, None, :])
             return fn1(x) * fn2(x)
@@ -104,7 +125,14 @@ def _quadrature_expectation(p, obj1, inducing_variable1, obj2, inducing_variable
 @dispatch.quadrature_expectation.register(
     MarkovGaussian, object, (InducingVariables, NoneType), object, (InducingVariables, NoneType)
 )
-def _quadrature_expectation(p, obj1, inducing_variable1, obj2, inducing_variable2, nghp=None):
+def _quadrature_expectation_markov(
+    p: MarkovGaussian,
+    obj1: ExpectationObject,
+    inducing_variable1: Optional[InducingVariables],
+    obj2: ExpectationObject,
+    inducing_variable2: Optional[InducingVariables],
+    nghp: Optional[int] = None,
+) -> tf.Tensor:
     """
     Handling of quadrature expectations for Markov Gaussians (useful for time series)
     Fallback method for missing analytic expectations wrt Markov Gaussians
@@ -121,19 +149,19 @@ def _quadrature_expectation(p, obj1, inducing_variable1, obj2, inducing_variable
 
     if obj2 is None:
 
-        def eval_func(x):
+        def eval_func(x: TensorType) -> tf.Tensor:
             return get_eval_func(obj1, inducing_variable1)(x)
 
         mu, cov = p.mu[:-1], p.cov[0, :-1]  # cross covariances are not needed
     elif obj1 is None:
 
-        def eval_func(x):
+        def eval_func(x: TensorType) -> tf.Tensor:
             return get_eval_func(obj2, inducing_variable2)(x)
 
         mu, cov = p.mu[1:], p.cov[0, 1:]  # cross covariances are not needed
     else:
 
-        def eval_func(x):
+        def eval_func(x: TensorType) -> tf.Tensor:
             x1 = tf.split(x, 2, 1)[0]
             x2 = tf.split(x, 2, 1)[1]
             res1 = get_eval_func(obj1, inducing_variable1, np.s_[:, :, None])(x1)
