@@ -12,13 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Any, Callable
+
 import numpy as np
 import tensorflow as tf
 
 from .. import logdensities
 from ..base import Parameter
 from ..utilities import positive
-from .base import ScalarLikelihood
+from .base import MeanAndVariance, ScalarLikelihood, TensorType
 from .utils import inv_probit
 
 
@@ -35,7 +37,12 @@ class Gaussian(ScalarLikelihood):
 
     DEFAULT_VARIANCE_LOWER_BOUND = 1e-6
 
-    def __init__(self, variance=1.0, variance_lower_bound=DEFAULT_VARIANCE_LOWER_BOUND, **kwargs):
+    def __init__(
+        self,
+        variance: float = 1.0,
+        variance_lower_bound: float = DEFAULT_VARIANCE_LOWER_BOUND,
+        **kwargs: Any,
+    ) -> None:
         """
         :param variance: The noise variance; must be greater than
             ``variance_lower_bound``.
@@ -51,22 +58,24 @@ class Gaussian(ScalarLikelihood):
 
         self.variance = Parameter(variance, transform=positive(lower=variance_lower_bound))
 
-    def _scalar_log_prob(self, F, Y):
+    def _scalar_log_prob(self, F: TensorType, Y: TensorType) -> tf.Tensor:
         return logdensities.gaussian(Y, F, self.variance)
 
-    def _conditional_mean(self, F):  # pylint: disable=R0201
+    def _conditional_mean(self, F: TensorType) -> tf.Tensor:  # pylint: disable=R0201
         return tf.identity(F)
 
-    def _conditional_variance(self, F):
+    def _conditional_variance(self, F: TensorType) -> tf.Tensor:
         return tf.fill(tf.shape(F), tf.squeeze(self.variance))
 
-    def _predict_mean_and_var(self, Fmu, Fvar):
+    def _predict_mean_and_var(self, Fmu: TensorType, Fvar: TensorType) -> MeanAndVariance:
         return tf.identity(Fmu), Fvar + self.variance
 
-    def _predict_log_density(self, Fmu, Fvar, Y):
+    def _predict_log_density(self, Fmu: TensorType, Fvar: TensorType, Y: TensorType) -> tf.Tensor:
         return tf.reduce_sum(logdensities.gaussian(Y, Fmu, Fvar + self.variance), axis=-1)
 
-    def _variational_expectations(self, Fmu, Fvar, Y):
+    def _variational_expectations(
+        self, Fmu: TensorType, Fvar: TensorType, Y: TensorType
+    ) -> tf.Tensor:
         return tf.reduce_sum(
             -0.5 * np.log(2 * np.pi)
             - 0.5 * tf.math.log(self.variance)
@@ -76,27 +85,29 @@ class Gaussian(ScalarLikelihood):
 
 
 class Exponential(ScalarLikelihood):
-    def __init__(self, invlink=tf.exp, **kwargs):
+    def __init__(self, invlink: Callable[[tf.Tensor], tf.Tensor] = tf.exp, **kwargs: Any) -> None:
         super().__init__(**kwargs)
         self.invlink = invlink
 
-    def _scalar_log_prob(self, F, Y):
+    def _scalar_log_prob(self, F: TensorType, Y: TensorType) -> tf.Tensor:
         return logdensities.exponential(Y, self.invlink(F))
 
-    def _conditional_mean(self, F):
+    def _conditional_mean(self, F: TensorType) -> tf.Tensor:
         return self.invlink(F)
 
-    def _conditional_variance(self, F):
+    def _conditional_variance(self, F: TensorType) -> tf.Tensor:
         return tf.square(self.invlink(F))
 
-    def _variational_expectations(self, Fmu, Fvar, Y):
+    def _variational_expectations(
+        self, Fmu: TensorType, Fvar: TensorType, Y: TensorType
+    ) -> tf.Tensor:
         if self.invlink is tf.exp:
             return tf.reduce_sum(-tf.exp(-Fmu + Fvar / 2) * Y - Fmu, axis=-1)
         return super()._variational_expectations(Fmu, Fvar, Y)
 
 
 class StudentT(ScalarLikelihood):
-    def __init__(self, scale=1.0, df=3.0, **kwargs):
+    def __init__(self, scale: float = 1.0, df: float = 3.0, **kwargs: Any) -> None:
         """
         :param scale float: scale parameter
         :param df float: degrees of freedom
@@ -105,13 +116,13 @@ class StudentT(ScalarLikelihood):
         self.df = df
         self.scale = Parameter(scale, transform=positive())
 
-    def _scalar_log_prob(self, F, Y):
+    def _scalar_log_prob(self, F: TensorType, Y: TensorType) -> tf.Tensor:
         return logdensities.student_t(Y, F, self.scale, self.df)
 
-    def _conditional_mean(self, F):
+    def _conditional_mean(self, F: TensorType) -> tf.Tensor:
         return F
 
-    def _conditional_variance(self, F):
+    def _conditional_variance(self, F: TensorType) -> tf.Tensor:
         var = (self.scale ** 2) * (self.df / (self.df - 2.0))
         return tf.fill(tf.shape(F), tf.squeeze(var))
 
@@ -121,22 +132,24 @@ class Gamma(ScalarLikelihood):
     Use the transformed GP to give the *scale* (inverse rate) of the Gamma
     """
 
-    def __init__(self, invlink=tf.exp, **kwargs):
+    def __init__(self, invlink: Callable[[tf.Tensor], tf.Tensor] = tf.exp, **kwargs: Any) -> None:
         super().__init__(**kwargs)
         self.invlink = invlink
         self.shape = Parameter(1.0, transform=positive())
 
-    def _scalar_log_prob(self, F, Y):
+    def _scalar_log_prob(self, F: TensorType, Y: TensorType) -> tf.Tensor:
         return logdensities.gamma(Y, self.shape, self.invlink(F))
 
-    def _conditional_mean(self, F):
+    def _conditional_mean(self, F: TensorType) -> tf.Tensor:
         return self.shape * self.invlink(F)
 
-    def _conditional_variance(self, F):
+    def _conditional_variance(self, F: TensorType) -> tf.Tensor:
         scale = self.invlink(F)
         return self.shape * (scale ** 2)
 
-    def _variational_expectations(self, Fmu, Fvar, Y):
+    def _variational_expectations(
+        self, Fmu: TensorType, Fvar: TensorType, Y: TensorType
+    ) -> tf.Tensor:
         if self.invlink is tf.exp:
             return tf.reduce_sum(
                 -self.shape * Fmu
@@ -166,20 +179,25 @@ class Beta(ScalarLikelihood):
         β  = scale * (1-m)
     """
 
-    def __init__(self, invlink=inv_probit, scale=1.0, **kwargs):
+    def __init__(
+        self,
+        invlink: Callable[[tf.Tensor], tf.Tensor] = inv_probit,
+        scale: float = 1.0,
+        **kwargs: Any,
+    ) -> None:
         super().__init__(**kwargs)
         self.scale = Parameter(scale, transform=positive())
         self.invlink = invlink
 
-    def _scalar_log_prob(self, F, Y):
+    def _scalar_log_prob(self, F: TensorType, Y: TensorType) -> tf.Tensor:
         mean = self.invlink(F)
         alpha = mean * self.scale
         beta = self.scale - alpha
         return logdensities.beta(Y, alpha, beta)
 
-    def _conditional_mean(self, F):
+    def _conditional_mean(self, F: TensorType) -> tf.Tensor:
         return self.invlink(F)
 
-    def _conditional_variance(self, F):
+    def _conditional_variance(self, F: TensorType) -> tf.Tensor:
         mean = self.invlink(F)
         return (mean - tf.square(mean)) / (self.scale + 1.0)
