@@ -13,18 +13,16 @@
 # limitations under the License.
 
 import abc
-from typing import Optional, Tuple
+from typing import Optional
 
 import tensorflow as tf
 
-from ..base import Module
+from ..base import InputData, MeanAndVariance, Module, RegressionData
 from ..conditionals.util import sample_mvn
 from ..kernels import Kernel, MultioutputKernel
 from ..likelihoods import Likelihood, SwitchedLikelihood
 from ..mean_functions import MeanFunction, Zero
-from ..types import MeanAndVariance
 from ..utilities import to_default_float
-from .training_mixins import InputData, RegressionData
 
 
 class BayesianModel(Module, metaclass=abc.ABCMeta):
