@@ -30,16 +30,13 @@ training data (InternalDataTrainingLossMixin), or requires it to be passed in
 to the objective function (ExternalDataTrainingLossMixin).
 """
 
-import abc
-from typing import Callable, Iterator, Optional, Tuple, TypeVar, Union
+from typing import Callable, TypeVar, Union
 
-import numpy as np
 import tensorflow as tf
 from tensorflow.python.data.ops.iterator_ops import OwnedIterator as DatasetOwnedIterator
 
-InputData = Union[tf.Tensor]
-OutputData = Union[tf.Tensor]
-RegressionData = Tuple[InputData, OutputData]
+from ..base import InputData, OutputData, RegressionData
+
 Data = TypeVar("Data", RegressionData, InputData, OutputData)
 
 
