@@ -14,12 +14,13 @@
 
 import enum
 from abc import ABC, abstractmethod
-from typing import Optional, Tuple, Union, cast
+from dataclasses import dataclass
+from typing import Optional, Tuple, Type, Union
 
 import tensorflow as tf
 
 from . import covariances, kernels, mean_functions
-from .base import Module, Parameter, TensorType
+from .base import MeanAndVariance, Module, Parameter, RegressionData, TensorType
 from .conditionals.util import (
     base_conditional,
     base_conditional_with_lm,
@@ -39,9 +40,10 @@ from .inducing_variables import (
     SeparateIndependentInducingVariables,
     SharedIndependentInducingVariables,
 )
-from .types import MeanAndVariance
+from .kernels import Kernel
+from .mean_functions import MeanFunction
 from .utilities import Dispatcher, add_noise_cov
-from .utilities.ops import leading_transpose
+from .utilities.ops import eye, leading_transpose
 
 
 class _QDistribution(Module):
@@ -52,22 +54,22 @@ class _QDistribution(Module):
 
 
 class _DeltaDist(_QDistribution):
-    def __init__(self, q_mu) -> None:
+    def __init__(self, q_mu: TensorType) -> None:
         self.q_mu = q_mu  # [M, L]
 
     @property
-    def q_sqrt(self):
+    def q_sqrt(self) -> Optional[tf.Tensor]:
         return None
 
 
 class _DiagNormal(_QDistribution):
-    def __init__(self, q_mu, q_sqrt) -> None:
+    def __init__(self, q_mu: TensorType, q_sqrt: TensorType) -> None:
         self.q_mu = q_mu  # [M, L]
         self.q_sqrt = q_sqrt  # [M, L]
 
 
 class _MvNormal(_QDistribution):
-    def __init__(self, q_mu, q_sqrt) -> None:
+    def __init__(self, q_mu: TensorType, q_sqrt: TensorType) -> None:
         self.q_mu = q_mu  # [M, L]
         self.q_sqrt = q_sqrt  # [L, M, M], lower-triangular
 
@@ -92,7 +94,60 @@ class PrecomputeCacheType(enum.Enum):
     NOCACHE = "nocache"
 
 
-def _validate_precompute_cache_type(value) -> PrecomputeCacheType:
+@dataclass
+class PrecomputedValue:
+    value: tf.Tensor
+    """
+    The precomputed value itself.
+    """
+
+    axis_dynamic: Tuple[bool, ...]
+    """
+    A tuple with one element per dimension of `value`. That element is `True` if that dimension
+    of `value` might change size.
+    """
+
+    def __post_init__(self) -> None:
+        tf.debugging.assert_rank(
+            self.value,
+            len(self.axis_dynamic),
+            "axis_dynamic must have one element per dimension of value.",
+        )
+
+    @staticmethod
+    def wrap_alpha_Qinv(alpha: TensorType, Qinv: TensorType) -> Tuple["PrecomputedValue", ...]:
+        """
+        Wraps `alpha` and `Qinv` in `PrecomputedValue`s.
+        """
+        one_dynamic = False
+        L_dynamic = False
+        M_dynamic = False  # TODO(jesper): Support variable number of inducing points?
+
+        alpha_rank = tf.rank(alpha)
+        if alpha_rank == 2:
+            alpha_dynamic: Tuple[bool, ...] = (M_dynamic, L_dynamic)
+        elif alpha_rank == 3:
+            alpha_dynamic = (L_dynamic, M_dynamic, one_dynamic)
+        else:
+            raise AssertionError(f"Unknown rank of alpha {alpha_rank}.")
+
+        Qinv_rank = tf.rank(Qinv)
+        if Qinv_rank == 2:
+            Qinv_dynamic: Tuple[bool, ...] = (M_dynamic, M_dynamic)
+        elif Qinv_rank == 3:
+            Qinv_dynamic = (L_dynamic, M_dynamic, M_dynamic)
+        else:
+            raise AssertionError(f"Unknown rank of Qinv {Qinv_rank}.")
+
+        return (
+            PrecomputedValue(alpha, alpha_dynamic),
+            PrecomputedValue(Qinv, Qinv_dynamic),
+        )
+
+
+def _validate_precompute_cache_type(
+    value: Union[None, PrecomputeCacheType, str]
+) -> PrecomputeCacheType:
     if value is None:
         return PrecomputeCacheType.NOCACHE
     elif isinstance(value, PrecomputeCacheType):
@@ -101,17 +156,17 @@ def _validate_precompute_cache_type(value) -> PrecomputeCacheType:
         return PrecomputeCacheType(value.lower())
     else:
         raise ValueError(
-            f"{value} is not a valid PrecomputeCacheType. Valid options: 'tensor', 'variable', 'nocache' (or None)."
+            f"{value} is not a valid PrecomputeCacheType."
+            " Valid options: 'tensor', 'variable', 'nocache' (or None)."
         )
 
 
 class AbstractPosterior(Module, ABC):
     def __init__(
         self,
-        kernel,
+        kernel: Kernel,
         X_data: Union[tf.Tensor, InducingVariables],
-        alpha: Optional[TensorType] = None,
-        Qinv: Optional[TensorType] = None,
+        cache: Optional[Tuple[tf.Tensor, ...]] = None,
         mean_function: Optional[mean_functions.MeanFunction] = None,
     ) -> None:
         """
@@ -125,18 +180,28 @@ class AbstractPosterior(Module, ABC):
 
         self.kernel = kernel
         self.X_data = X_data
-        self.alpha = alpha
-        self.Qinv = Qinv
+        self.cache = cache
         self.mean_function = mean_function
 
-    def _add_mean_function(self, Xnew, mean) -> tf.Tensor:
+        self._precompute_cache: Optional[PrecomputeCacheType] = None
+
+    def _add_mean_function(self, Xnew: TensorType, mean: TensorType) -> tf.Tensor:
         if self.mean_function is None:
             return mean
         else:
             return mean + self.mean_function(Xnew)
 
+    @abstractmethod
+    def _precompute(self) -> Tuple[PrecomputedValue, ...]:
+        """
+        Precompute a cache.
+
+        The result of this method will later be passed to `_conditional_with_precompute` as the
+        `cache` argument.
+        """
+
     def fused_predict_f(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         """
         Computes predictive mean and (co)variance at Xnew, including mean_function
@@ -149,7 +214,7 @@ class AbstractPosterior(Module, ABC):
 
     @abstractmethod
     def _conditional_fused(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         """
         Computes predictive mean and (co)variance at Xnew, *excluding* mean_function
@@ -157,24 +222,28 @@ class AbstractPosterior(Module, ABC):
         """
 
     def predict_f(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         """
         Computes predictive mean and (co)variance at Xnew, including mean_function.
         Relies on precomputed alpha and Qinv (see _precompute method)
         """
-        if self.alpha is None or self.Qinv is None:
+        if self.cache is None:
             raise ValueError(
                 "Cache has not been precomputed yet. Call update_cache first or use fused_predict_f"
             )
         mean, cov = self._conditional_with_precompute(
-            Xnew, full_cov=full_cov, full_output_cov=full_output_cov
+            self.cache, Xnew, full_cov=full_cov, full_output_cov=full_output_cov
         )
         return self._add_mean_function(Xnew, mean), cov
 
     @abstractmethod
     def _conditional_with_precompute(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self,
+        cache: Tuple[tf.Tensor, ...],
+        Xnew: TensorType,
+        full_cov: bool = False,
+        full_output_cov: bool = False,
     ) -> MeanAndVariance:
         """
         Computes predictive mean and (co)variance at Xnew, *excluding* mean_function.
@@ -188,48 +257,50 @@ class AbstractPosterior(Module, ABC):
         is not given, the setting defaults to the most-recently-used one.
         """
         if precompute_cache is None:
-            try:
-                precompute_cache = cast(
-                    PrecomputeCacheType,
-                    self._precompute_cache,  # type: ignore
-                )
-            except AttributeError:
+            if self._precompute_cache is None:
                 raise ValueError(
-                    "You must pass precompute_cache explicitly (the cache had not been updated before)."
+                    "You must pass precompute_cache explicitly"
+                    " (the cache had not been updated before)."
                 )
+            precompute_cache = self._precompute_cache
         else:
             self._precompute_cache = precompute_cache
 
         if precompute_cache is PrecomputeCacheType.NOCACHE:
-            self.alpha = self.Qinv = None
+            self.cache = None
 
         elif precompute_cache is PrecomputeCacheType.TENSOR:
-            self.alpha, self.Qinv = self._precompute()
+            self.cache = tuple(c.value for c in self._precompute())
 
         elif precompute_cache is PrecomputeCacheType.VARIABLE:
-            alpha, Qinv = self._precompute()
-            if isinstance(self.alpha, tf.Variable) and isinstance(self.Qinv, tf.Variable):
+            cache = self._precompute()
+
+            if self.cache is not None and all(isinstance(c, tf.Variable) for c in self.cache):
                 # re-use existing variables
-                self.alpha.assign(alpha)
-                self.Qinv.assign(Qinv)
+                for cache_var, c in zip(self.cache, cache):
+                    cache_var.assign(c.value)
             else:  # create variables
-                self.alpha = tf.Variable(alpha, trainable=False)
-                self.Qinv = tf.Variable(Qinv, trainable=False)
+                shapes = [
+                    [None if d else s for d, s in zip(c.axis_dynamic, tf.shape(c.value))]
+                    for c in cache
+                ]
+                self.cache = tuple(
+                    tf.Variable(c.value, trainable=False, shape=s) for c, s in zip(cache, shapes)
+                )
 
 
 class GPRPosterior(AbstractPosterior):
     def __init__(
         self,
-        kernel,
-        data: Tuple[tf.Tensor, tf.Tensor],
+        kernel: Kernel,
+        data: RegressionData,
         likelihood_variance: Parameter,
-        mean_function: Optional[mean_functions.MeanFunction] = None,
+        mean_function: MeanFunction,
         *,
         precompute_cache: Optional[PrecomputeCacheType],
     ) -> None:
         X, Y = data
         super().__init__(kernel, X, mean_function=mean_function)
-        self.mean_function = mean_function
         self.Y_data = Y
         self.likelihood_variance = likelihood_variance
 
@@ -237,12 +308,18 @@ class GPRPosterior(AbstractPosterior):
             self.update_cache(precompute_cache)
 
     def _conditional_with_precompute(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self,
+        cache: Tuple[tf.Tensor, ...],
+        Xnew: TensorType,
+        full_cov: bool = False,
+        full_output_cov: bool = False,
     ) -> MeanAndVariance:
         """
         Computes predictive mean and (co)variance at Xnew, *excluding* mean_function.
         Relies on cached alpha and Qinv.
         """
+        (alpha,) = cache
+        (Qinv,) = cache
 
         Kmn = self.kernel(self.X_data, Xnew)
         # compute kernel stuff
@@ -265,31 +342,37 @@ class GPRPosterior(AbstractPosterior):
         # get the leading dims in Knm to the front of the tensor Knm
         Knm = leading_transpose(Kmn, [..., -1, -2])
 
+        assert self.mean_function is not None
         Knn = self.kernel(Xnew, full_cov=full_cov)
-        err = self.Y_data - self.mean_function(self.X_data)  # type: ignore
+        err = self.Y_data - self.mean_function(self.X_data)
 
-        mean = Knm @ self.alpha @ err
+        mean = Knm @ alpha @ err
 
         # The GPR model only has a single latent GP.
         if full_cov:
-            cov = Knn - Knm @ self.Qinv @ Kmn  # [..., N, N]
+            cov = Knn - Knm @ Qinv @ Kmn  # [..., N, N]
             cov_shape = tf.concat([leading_dims, [num_func, N, N]], 0)
             cov = tf.broadcast_to(tf.expand_dims(cov, -3), cov_shape)  # [..., R, N, N]
 
         else:
-            cov = Knn - tf.linalg.diag_part(Knm @ self.Qinv @ Kmn)  # [..., N]
+            cov = Knn - tf.linalg.diag_part(Knm @ Qinv @ Kmn)  # [..., N]
             cov_shape = tf.concat([leading_dims, [num_func, N]], 0)  # [..., R, N]
             cov = tf.broadcast_to(tf.expand_dims(cov, -2), cov_shape)  # [..., R, N]
             cov = tf.linalg.adjoint(cov)
 
         return mean, cov
 
-    def _precompute(self) -> Tuple[tf.Tensor, tf.Tensor]:
+    def _precompute(self) -> Tuple[PrecomputedValue, ...]:
         Kmm = self.kernel(self.X_data)
         Kmm_plus_s = add_noise_cov(Kmm, self.likelihood_variance)
 
         Lm = tf.linalg.cholesky(Kmm_plus_s)
-        Kmm_plus_s_inv = tf.linalg.cholesky_solve(Lm, tf.eye(self.X_data.shape[0], dtype=Lm.dtype))
+        Kmm_plus_s_inv = tf.linalg.cholesky_solve(
+            Lm, tf.eye(tf.shape(self.X_data)[0], dtype=Lm.dtype)
+        )
+
+        M = self.X_data.shape[0]
+        M_dynamic = M is None
 
         tf.debugging.assert_shapes(
             [
@@ -297,10 +380,10 @@ class GPRPosterior(AbstractPosterior):
                 (Kmm, ["M", "M"]),
             ]
         )
-        return Kmm_plus_s_inv, Kmm_plus_s_inv
+        return (PrecomputedValue(Kmm_plus_s_inv, (M_dynamic, M_dynamic)),)
 
     def _conditional_fused(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         """
         Computes predictive mean and (co)variance at Xnew, *excluding* mean_function
@@ -308,7 +391,8 @@ class GPRPosterior(AbstractPosterior):
         """
 
         # taken directly from the deprecated GPR implementation
-        err = self.Y_data - self.mean_function(self.X_data)  # type: ignore
+        assert self.mean_function is not None
+        err = self.Y_data - self.mean_function(self.X_data)
 
         Kmm = self.kernel(self.X_data)
         Knn = self.kernel(Xnew, full_cov=full_cov)
@@ -328,18 +412,17 @@ class SGPRPosterior(AbstractPosterior):
 
     def __init__(
         self,
-        kernel,
-        data: Tuple[tf.Tensor, tf.Tensor],
+        kernel: Kernel,
+        data: RegressionData,
         inducing_variable: InducingPoints,
         likelihood_variance: Parameter,
         num_latent_gps: int,
-        mean_function: Optional[mean_functions.MeanFunction] = None,
+        mean_function: MeanFunction,
         *,
         precompute_cache: Optional[PrecomputeCacheType],
     ) -> None:
         X, Y = data
         super().__init__(kernel, X, mean_function=mean_function)
-        self.mean_function = mean_function
         self.Y_data = Y
         self.likelihood_variance = likelihood_variance
         self.inducing_variable = inducing_variable
@@ -348,10 +431,11 @@ class SGPRPosterior(AbstractPosterior):
         if precompute_cache is not None:
             self.update_cache(precompute_cache)
 
-    def _precompute(self) -> Tuple[tf.Tensor, tf.Tensor]:
+    def _precompute(self) -> Tuple[PrecomputedValue, ...]:
         # taken directly from the deprecated SGPR implementation
         num_inducing = self.inducing_variable.num_inducing
-        err = self.Y_data - self.mean_function(self.X_data)  # type:ignore
+        assert self.mean_function is not None
+        err = self.Y_data - self.mean_function(self.X_data)
         kuf = Kuf(self.inducing_variable, self.kernel, self.X_data)
         kuu = Kuu(self.inducing_variable, self.kernel, jitter=default_jitter())
         sigma = tf.sqrt(self.likelihood_variance)
@@ -375,33 +459,39 @@ class SGPRPosterior(AbstractPosterior):
         alpha = LinvT @ tf.transpose(LBinv) @ c
         Qinv = LinvT @ tmp @ Linv
 
-        return alpha, Qinv
+        return PrecomputedValue.wrap_alpha_Qinv(alpha, Qinv)
 
     def _conditional_with_precompute(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self,
+        cache: Tuple[tf.Tensor, ...],
+        Xnew: TensorType,
+        full_cov: bool = False,
+        full_output_cov: bool = False,
     ) -> MeanAndVariance:
         """
         Computes predictive mean and (co)variance at Xnew, *excluding* mean_function.
         Relies on cached alpha and Qinv.
         """
+        alpha, Qinv = cache
+
         Kus = Kuf(self.inducing_variable, self.kernel, Xnew)
         Knn = self.kernel(Xnew, full_cov=full_cov)
 
         Ksu = tf.transpose(Kus)
-        mean = Ksu @ self.alpha
+        mean = Ksu @ alpha
 
         if full_cov:
-            var = Knn - Ksu @ self.Qinv @ Kus
+            var = Knn - Ksu @ Qinv @ Kus
             var = tf.tile(var[None, ...], [self.num_latent_gps, 1, 1])  # [P, N, N]
         else:
-            Kfu_Qinv_Kuf = tf.reduce_sum(Kus * tf.matmul(self.Qinv, Kus), axis=-2)
+            Kfu_Qinv_Kuf = tf.reduce_sum(Kus * tf.matmul(Qinv, Kus), axis=-2)
             var = Knn - Kfu_Qinv_Kuf
             var = tf.tile(var[:, None], [1, self.num_latent_gps])
 
         return mean, var
 
     def _conditional_fused(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         """
         Compute the mean and variance of the latent function at some new points
@@ -410,11 +500,12 @@ class SGPRPosterior(AbstractPosterior):
 
         # taken directly from the deprecated SGPR implementation
         num_inducing = self.inducing_variable.num_inducing
-        err = self.Y_data - self.mean_function(self.X_data)  # type: ignore
+        assert self.mean_function is not None
+        err = self.Y_data - self.mean_function(self.X_data)
         kuf = Kuf(self.inducing_variable, self.kernel, self.X_data)
         kuu = Kuu(self.inducing_variable, self.kernel, jitter=default_jitter())
         Kus = Kuf(self.inducing_variable, self.kernel, Xnew)
-        sigma = tf.sqrt(self.likelihood_variance)  # type: ignore
+        sigma = tf.sqrt(self.likelihood_variance)
         L = tf.linalg.cholesky(kuu)  # cache alpha, qinv
         A = tf.linalg.triangular_solve(L, kuf, lower=True) / sigma
         B = tf.linalg.matmul(A, A, transpose_b=True) + tf.eye(
@@ -444,11 +535,72 @@ class SGPRPosterior(AbstractPosterior):
         return mean, var
 
 
+class VGPPosterior(AbstractPosterior):
+    def __init__(
+        self,
+        kernel: Kernel,
+        X: tf.Tensor,
+        q_mu: tf.Tensor,
+        q_sqrt: tf.Tensor,
+        mean_function: Optional[mean_functions.MeanFunction] = None,
+        white: bool = True,
+        *,
+        precompute_cache: Optional[PrecomputeCacheType],
+    ) -> None:
+        super().__init__(kernel, X, mean_function=mean_function)
+        self.q_mu = q_mu
+        self.q_sqrt = q_sqrt
+        self.white = white
+
+        if precompute_cache is not None:
+            self.update_cache(precompute_cache)
+
+    def _conditional_with_precompute(
+        self,
+        cache: Tuple[tf.Tensor, ...],
+        Xnew: TensorType,
+        full_cov: bool = False,
+        full_output_cov: bool = False,
+    ) -> MeanAndVariance:
+        (Lm,) = cache
+        Kmn = self.kernel(self.X_data, Xnew)  # [M, ..., N]
+        Knn = self.kernel(
+            Xnew, full_cov=full_cov
+        )  # [..., N] (full_cov = False) or [..., N, N] (True)
+
+        return base_conditional_with_lm(
+            Kmn=Kmn,
+            Lm=Lm,
+            Knn=Knn,
+            f=self.q_mu,
+            full_cov=full_cov,
+            q_sqrt=self.q_sqrt,
+            white=self.white,
+        )
+
+    def _precompute(self) -> Tuple[PrecomputedValue, ...]:
+        Kmm = self.kernel(self.X_data) + eye(
+            tf.shape(self.X_data)[-2], value=default_jitter(), dtype=self.X_data.dtype
+        )  # [..., M, M]
+        Lm = tf.linalg.cholesky(Kmm)
+
+        M = self.X_data.shape[0]
+        M_dynamic = M is None
+
+        return (PrecomputedValue(Lm, (M_dynamic, M_dynamic)),)
+
+    def _conditional_fused(
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
+    ) -> MeanAndVariance:
+        temp_cache = tuple(c.value for c in self._precompute())
+        return self._conditional_with_precompute(temp_cache, Xnew, full_cov, full_output_cov)
+
+
 class BasePosterior(AbstractPosterior):
     def __init__(
         self,
-        kernel,
-        inducing_variable,
+        kernel: Kernel,
+        inducing_variable: InducingVariables,
         q_mu: tf.Tensor,
         q_sqrt: tf.Tensor,
         whiten: bool = True,
@@ -465,14 +617,14 @@ class BasePosterior(AbstractPosterior):
             self.update_cache(precompute_cache)
 
     @property
-    def q_mu(self):
+    def q_mu(self) -> tf.Tensor:
         return self._q_dist.q_mu
 
     @property
-    def q_sqrt(self):
+    def q_sqrt(self) -> tf.Tensor:
         return self._q_dist.q_sqrt
 
-    def _set_qdist(self, q_mu, q_sqrt):
+    def _set_qdist(self, q_mu: TensorType, q_sqrt: TensorType) -> tf.Tensor:
         if q_sqrt is None:
             self._q_dist = _DeltaDist(q_mu)
         elif len(q_sqrt.shape) == 2:  # q_diag
@@ -480,7 +632,7 @@ class BasePosterior(AbstractPosterior):
         else:
             self._q_dist = _MvNormal(q_mu, q_sqrt)
 
-    def _precompute(self):
+    def _precompute(self) -> Tuple[PrecomputedValue, ...]:
         Kuu = covariances.Kuu(self.X_data, self.kernel, jitter=default_jitter())  # [(R), M, M]
         q_mu = self._q_dist.q_mu
 
@@ -538,26 +690,30 @@ class BasePosterior(AbstractPosterior):
             ]
         )
 
-        return alpha, Qinv
+        return PrecomputedValue.wrap_alpha_Qinv(alpha, Qinv)
 
 
 class IndependentPosterior(BasePosterior):
-    def _post_process_mean_and_cov(self, mean, cov, full_cov, full_output_cov):
+    def _post_process_mean_and_cov(
+        self, mean: TensorType, cov: TensorType, full_cov: bool, full_output_cov: bool
+    ) -> tf.Tensor:
         return mean, expand_independent_outputs(cov, full_cov, full_output_cov)
 
-    def _get_Kff(self, Xnew, full_cov):
+    def _get_Kff(self, Xnew: TensorType, full_cov: bool) -> tf.Tensor:
 
         # TODO: this assumes that Xnew has shape [N, D] and no leading dims
 
         if isinstance(self.kernel, (kernels.SeparateIndependent, kernels.IndependentLatent)):
-            # NOTE calling kernel(Xnew, full_cov=full_cov, full_output_cov=False) directly would return
+            # NOTE calling kernel(Xnew, full_cov=full_cov, full_output_cov=False) directly would
+            # return
             # if full_cov: [P, N, N] -- this is what we want
             # else: [N, P] instead of [P, N] as we get from the explicit stack below
             Kff = tf.stack([k(Xnew, full_cov=full_cov) for k in self.kernel.kernels], axis=0)
         elif isinstance(self.kernel, kernels.MultioutputKernel):
             # effectively, SharedIndependent path
             Kff = self.kernel.kernel(Xnew, full_cov=full_cov)
-            # NOTE calling kernel(Xnew, full_cov=full_cov, full_output_cov=False) directly would return
+            # NOTE calling kernel(Xnew, full_cov=full_cov, full_output_cov=False) directly would
+            # return
             # if full_cov: [P, N, N] instead of [N, N]
             # else: [N, P] instead of [N]
         else:
@@ -567,25 +723,30 @@ class IndependentPosterior(BasePosterior):
         return Kff
 
     def _conditional_with_precompute(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self,
+        cache: Tuple[tf.Tensor, ...],
+        Xnew: TensorType,
+        full_cov: bool = False,
+        full_output_cov: bool = False,
     ) -> MeanAndVariance:
         # Qinv: [L, M, M]
         # alpha: [M, L]
+        alpha, Qinv = cache
 
         Kuf = covariances.Kuf(self.X_data, self.kernel, Xnew)  # [(R), M, N]
         Kff = self._get_Kff(Xnew, full_cov)
 
-        mean = tf.matmul(Kuf, self.alpha, transpose_a=True)
+        mean = tf.matmul(Kuf, alpha, transpose_a=True)
         if Kuf.shape.ndims == 3:
             mean = tf.linalg.adjoint(tf.squeeze(mean, axis=-1))
 
         if full_cov:
-            Kfu_Qinv_Kuf = tf.matmul(Kuf, self.Qinv @ Kuf, transpose_a=True)
+            Kfu_Qinv_Kuf = tf.matmul(Kuf, Qinv @ Kuf, transpose_a=True)
             cov = Kff - Kfu_Qinv_Kuf
         else:
             # [Aᵀ B]_ij = Aᵀ_ik B_kj = A_ki B_kj
             # TODO check whether einsum is faster now?
-            Kfu_Qinv_Kuf = tf.reduce_sum(Kuf * tf.matmul(self.Qinv, Kuf), axis=-2)
+            Kfu_Qinv_Kuf = tf.reduce_sum(Kuf * tf.matmul(Qinv, Kuf), axis=-2)
             cov = Kff - Kfu_Qinv_Kuf
             cov = tf.linalg.adjoint(cov)
 
@@ -595,9 +756,10 @@ class IndependentPosterior(BasePosterior):
 class IndependentPosteriorSingleOutput(IndependentPosterior):
     # could almost be the same as IndependentPosteriorMultiOutput ...
     def _conditional_fused(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
-        # same as IndependentPosteriorMultiOutput, Shared~/Shared~ branch, except for following line:
+        # same as IndependentPosteriorMultiOutput, Shared~/Shared~ branch, except for following
+        # line:
         Knn = self.kernel(Xnew, full_cov=full_cov)
 
         Kmm = covariances.Kuu(self.X_data, self.kernel, jitter=default_jitter())  # [M, M]
@@ -611,7 +773,7 @@ class IndependentPosteriorSingleOutput(IndependentPosterior):
 
 class IndependentPosteriorMultiOutput(IndependentPosterior):
     def _conditional_fused(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         if isinstance(self.X_data, SharedIndependentInducingVariables) and isinstance(
             self.kernel, kernels.SharedIndependent
@@ -627,7 +789,8 @@ class IndependentPosteriorMultiOutput(IndependentPosterior):
                 Kmn, Kmm, Knn, self.q_mu, full_cov=full_cov, q_sqrt=self.q_sqrt, white=self.whiten
             )  # [N, P],  [P, N, N] or [N, P]
         else:
-            # this is the messy thing with tf.map_fn, cleaned up by the st/clean_up_broadcasting_conditionals branch
+            # this is the messy thing with tf.map_fn, cleaned up by the
+            # st/clean_up_broadcasting_conditionals branch
 
             # Following are: [P, M, M]  -  [P, M, N]  -  [P, N](x N)
             Kmms = covariances.Kuu(self.X_data, self.kernel, jitter=default_jitter())  # [P, M, M]
@@ -654,7 +817,9 @@ class IndependentPosteriorMultiOutput(IndependentPosterior):
 
 
 class LinearCoregionalizationPosterior(IndependentPosteriorMultiOutput):
-    def _post_process_mean_and_cov(self, mean, cov, full_cov, full_output_cov):
+    def _post_process_mean_and_cov(
+        self, mean: TensorType, cov: TensorType, full_cov: bool, full_output_cov: bool
+    ) -> MeanAndVariance:
         """
         mean: [N, L]
         cov: [L, N, N] or [N, L]
@@ -666,19 +831,25 @@ class LinearCoregionalizationPosterior(IndependentPosteriorMultiOutput):
 
 class FullyCorrelatedPosterior(BasePosterior):
     def _conditional_with_precompute(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self,
+        cache: Tuple[tf.Tensor, ...],
+        Xnew: TensorType,
+        full_cov: bool = False,
+        full_output_cov: bool = False,
     ) -> MeanAndVariance:
         # TODO: this assumes that Xnew has shape [N, D] and no leading dims
 
         # Qinv: [L, M, M]
         # alpha: [M, L]
+        alpha, Qinv = cache
 
         Kuf = covariances.Kuf(self.X_data, self.kernel, Xnew)
         assert Kuf.shape.ndims == 4
         M, L, N, K = tf.unstack(tf.shape(Kuf), num=Kuf.shape.ndims, axis=0)
         Kuf = tf.reshape(Kuf, (M * L, N * K))
 
-        Kff = self.kernel(Xnew, full_cov=full_cov, full_output_cov=full_output_cov)
+        kernel: kernels.MultioutputKernel = self.kernel
+        Kff = kernel(Xnew, full_cov=full_cov, full_output_cov=full_output_cov)
         # full_cov=True and full_output_cov=True: [N, P, N, P]
         # full_cov=True and full_output_cov=False: [P, N, N]
         # full_cov=False and full_output_cov=True: [N, P, P]
@@ -690,7 +861,7 @@ class FullyCorrelatedPosterior(BasePosterior):
         N = tf.shape(Xnew)[0]
         K = tf.shape(Kuf)[-1] // N
 
-        mean = tf.matmul(Kuf, self.alpha, transpose_a=True)
+        mean = tf.matmul(Kuf, alpha, transpose_a=True)
         if Kuf.shape.ndims == 3:
             mean = tf.linalg.adjoint(tf.squeeze(mean, axis=-1))
 
@@ -698,9 +869,9 @@ class FullyCorrelatedPosterior(BasePosterior):
             # fully diagonal case in both inputs and outputs
             # [Aᵀ B]_ij = Aᵀ_ik B_kj = A_ki B_kj
             # TODO check whether einsum is faster now?
-            Kfu_Qinv_Kuf = tf.reduce_sum(Kuf * tf.matmul(self.Qinv, Kuf), axis=-2)
+            Kfu_Qinv_Kuf = tf.reduce_sum(Kuf * tf.matmul(Qinv, Kuf), axis=-2)
         else:
-            Kfu_Qinv_Kuf = tf.matmul(Kuf, self.Qinv @ Kuf, transpose_a=True)
+            Kfu_Qinv_Kuf = tf.matmul(Kuf, Qinv @ Kuf, transpose_a=True)
             if not (full_cov and full_output_cov):
                 # diagonal in either inputs or outputs
                 new_shape = tf.concat([tf.shape(Kfu_Qinv_Kuf)[:-2], (N, K, N, K)], axis=0)
@@ -727,11 +898,12 @@ class FullyCorrelatedPosterior(BasePosterior):
         return mean, cov
 
     def _conditional_fused(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         Kmm = covariances.Kuu(self.X_data, self.kernel, jitter=default_jitter())  # [M, L, M, L]
         Kmn = covariances.Kuf(self.X_data, self.kernel, Xnew)  # [M, L, N, P]
-        Knn = self.kernel(
+        kernel: kernels.MultioutputKernel = self.kernel
+        Knn = kernel(
             Xnew, full_cov=full_cov, full_output_cov=full_output_cov
         )  # [N, P](x N)x P  or  [N, P](x P)
 
@@ -763,11 +935,12 @@ class FullyCorrelatedPosterior(BasePosterior):
 
 class FallbackIndependentLatentPosterior(FullyCorrelatedPosterior):  # XXX
     def _conditional_fused(
-        self, Xnew, full_cov: bool = False, full_output_cov: bool = False
+        self, Xnew: TensorType, full_cov: bool = False, full_output_cov: bool = False
     ) -> MeanAndVariance:
         Kmm = covariances.Kuu(self.X_data, self.kernel, jitter=default_jitter())  # [L, M, M]
         Kmn = covariances.Kuf(self.X_data, self.kernel, Xnew)  # [M, L, N, P]
-        Knn = self.kernel(
+        kernel: kernels.IndependentLatent = self.kernel
+        Knn = kernel(
             Xnew, full_cov=full_cov, full_output_cov=full_output_cov
         )  # [N, P](x N)x P  or  [N, P](x P)
 
@@ -787,13 +960,17 @@ get_posterior_class = Dispatcher("get_posterior_class")
 
 
 @get_posterior_class.register(kernels.Kernel, InducingVariables)
-def _get_posterior_base_case(kernel, inducing_variable):
+def _get_posterior_base_case(
+    kernel: Kernel, inducing_variable: InducingVariables
+) -> Type[AbstractPosterior]:
     # independent single output
     return IndependentPosteriorSingleOutput
 
 
 @get_posterior_class.register(kernels.MultioutputKernel, InducingPoints)
-def _get_posterior_fully_correlated_mo(kernel, inducing_variable):
+def _get_posterior_fully_correlated_mo(
+    kernel: Kernel, inducing_variable: InducingVariables
+) -> Type[AbstractPosterior]:
     return FullyCorrelatedPosterior
 
 
@@ -801,7 +978,9 @@ def _get_posterior_fully_correlated_mo(kernel, inducing_variable):
     (kernels.SharedIndependent, kernels.SeparateIndependent),
     (SeparateIndependentInducingVariables, SharedIndependentInducingVariables),
 )
-def _get_posterior_independent_mo(kernel, inducing_variable):
+def _get_posterior_independent_mo(
+    kernel: Kernel, inducing_variable: InducingVariables
+) -> Type[AbstractPosterior]:
     # independent multi-output
     return IndependentPosteriorMultiOutput
 
@@ -810,7 +989,9 @@ def _get_posterior_independent_mo(kernel, inducing_variable):
     kernels.IndependentLatent,
     (FallbackSeparateIndependentInducingVariables, FallbackSharedIndependentInducingVariables),
 )
-def _get_posterior_independentlatent_mo_fallback(kernel, inducing_variable):
+def _get_posterior_independentlatent_mo_fallback(
+    kernel: Kernel, inducing_variable: InducingVariables
+) -> Type[AbstractPosterior]:
     return FallbackIndependentLatentPosterior
 
 
@@ -818,20 +999,22 @@ def _get_posterior_independentlatent_mo_fallback(kernel, inducing_variable):
     kernels.LinearCoregionalization,
     (SeparateIndependentInducingVariables, SharedIndependentInducingVariables),
 )
-def _get_posterior_linearcoregionalization_mo_efficient(kernel, inducing_variable):
+def _get_posterior_linearcoregionalization_mo_efficient(
+    kernel: Kernel, inducing_variable: InducingVariables
+) -> Type[AbstractPosterior]:
     # Linear mixing---efficient multi-output
     return LinearCoregionalizationPosterior
 
 
 def create_posterior(
-    kernel,
-    inducing_variable,
-    q_mu,
-    q_sqrt,
-    whiten,
-    mean_function=None,
+    kernel: Kernel,
+    inducing_variable: InducingVariables,
+    q_mu: TensorType,
+    q_sqrt: TensorType,
+    whiten: bool,
+    mean_function: Optional[MeanFunction] = None,
     precompute_cache: Union[PrecomputeCacheType, str, None] = PrecomputeCacheType.TENSOR,
-):
+) -> AbstractPosterior:
     posterior_class = get_posterior_class(kernel, inducing_variable)
     precompute_cache = _validate_precompute_cache_type(precompute_cache)
     return posterior_class(
