@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Any, Optional, Sequence, Tuple, Type, Union
+
 import numpy as np
 import pytest
 import tensorflow as tf
@@ -19,14 +21,31 @@ from numpy.testing import assert_allclose
 
 import gpflow
 import gpflow.ci_utils
+from gpflow.base import TensorType
 from gpflow.config import default_float
 from gpflow.kernels import (
+    RBF,
+    AnisotropicStationary,
     ArcCosine,
+    ChangePoints,
+    Constant,
+    Convolutional,
+    Coregion,
+    IsotropicStationary,
+    Kernel,
     Linear,
     LinearCoregionalization,
+    Matern12,
+    Matern32,
+    Matern52,
+    MultioutputKernel,
+    Periodic,
+    Polynomial,
+    RationalQuadratic,
     SeparateIndependent,
     SharedIndependent,
     SquaredExponential,
+    Stationary,
     White,
 )
 from tests.gpflow.kernels.reference import ref_arccosine_kernel, ref_periodic_kernel, ref_rbf_kernel
@@ -34,18 +53,24 @@ from tests.gpflow.kernels.reference import ref_arccosine_kernel, ref_periodic_ke
 rng = np.random.RandomState(1)
 
 
-def _ref_changepoints(X, kernels, locations, steepness):
+def _ref_changepoints(
+    X: np.ndarray,
+    kernels: Sequence[Kernel],
+    locations: Sequence[float],
+    steepness: Union[float, Sequence[float]],
+) -> np.ndarray:
     """
     Calculates K(X) for each kernel in `kernels`, then multiply by sigmoid functions
     in order to smoothly transition betwen them. The sigmoid transitions are defined
     by a location and a steepness parameter.
     """
     locations = sorted(locations)
-    steepness = steepness if isinstance(steepness, list) else [steepness] * len(locations)
+    steepness = steepness if isinstance(steepness, Sequence) else [steepness] * len(locations)
     locations = np.array(locations).reshape((1, 1, -1))
     steepness = np.array(steepness).reshape((1, 1, -1))
 
-    sig_X = 1.0 / (1.0 + np.exp(-steepness * (X[:, :, None] - locations)))
+    # type-ignore, because numpy handles casting from lists.
+    sig_X = 1.0 / (1.0 + np.exp(-steepness * (X[:, :, None] - locations)))  # type: ignore
 
     starters = sig_X * np.transpose(sig_X, axes=(1, 0, 2))
     stoppers = (1 - sig_X) * np.transpose((1 - sig_X), axes=(1, 0, 2))
@@ -59,9 +84,9 @@ def _ref_changepoints(X, kernels, locations, steepness):
 
 
 @pytest.mark.parametrize("variance, lengthscales", [[2.3, 1.4]])
-def test_rbf_1d(variance, lengthscales):
+def test_rbf_1d(variance: TensorType, lengthscales: TensorType) -> None:
     X = rng.randn(3, 1)
-    kernel = gpflow.kernels.SquaredExponential(lengthscales=lengthscales, variance=variance)
+    kernel = SquaredExponential(lengthscales=lengthscales, variance=variance)
 
     gram_matrix = kernel(X)
     reference_gram_matrix = ref_rbf_kernel(X, lengthscales, variance)
@@ -70,9 +95,9 @@ def test_rbf_1d(variance, lengthscales):
 
 
 @pytest.mark.parametrize("variance, lengthscales", [[2.3, 1.4]])
-def test_rq_1d(variance, lengthscales):
-    kSE = gpflow.kernels.SquaredExponential(lengthscales=lengthscales, variance=variance)
-    kRQ = gpflow.kernels.RationalQuadratic(lengthscales=lengthscales, variance=variance, alpha=1e8)
+def test_rq_1d(variance: TensorType, lengthscales: TensorType) -> None:
+    kSE = SquaredExponential(lengthscales=lengthscales, variance=variance)
+    kRQ = RationalQuadratic(lengthscales=lengthscales, variance=variance, alpha=1e8)
     rng = np.random.RandomState(1)
     X = rng.randn(6, 1).astype(default_float())
 
@@ -81,8 +106,14 @@ def test_rq_1d(variance, lengthscales):
     assert_allclose(gram_matrix_SE, gram_matrix_RQ)
 
 
-def _assert_arccosine_kern_err(variance, weight_variances, bias_variance, order, X):
-    kernel = gpflow.kernels.ArcCosine(
+def _assert_arccosine_kern_err(
+    variance: TensorType,
+    weight_variances: TensorType,
+    bias_variance: TensorType,
+    order: int,
+    X: TensorType,
+) -> None:
+    kernel = ArcCosine(
         order=order,
         variance=variance,
         weight_variances=weight_variances,
@@ -95,24 +126,31 @@ def _assert_arccosine_kern_err(variance, weight_variances, bias_variance, order,
     assert_allclose(gram_matrix, reference_gram_matrix)
 
 
-@pytest.mark.parametrize("order", gpflow.kernels.ArcCosine.implemented_orders)
+@pytest.mark.parametrize("order", ArcCosine.implemented_orders)
 @pytest.mark.parametrize("D, weight_variances", [[1, 1.7], [3, 1.7], [3, (1.1, 1.7, 1.9)]])
 @pytest.mark.parametrize("N, bias_variance, variance", [[3, 0.6, 2.3]])
-def test_arccosine_1d_and_3d(order, D, N, weight_variances, bias_variance, variance):
+def test_arccosine_1d_and_3d(
+    order: int,
+    D: int,
+    N: int,
+    weight_variances: Union[float, Sequence[float]],
+    bias_variance: float,
+    variance: float,
+) -> None:
     X_data = rng.randn(N, D)
     _assert_arccosine_kern_err(variance, weight_variances, bias_variance, order, X_data)
 
 
 @pytest.mark.parametrize("order", [42])
-def test_arccosine_non_implemented_order(order):
+def test_arccosine_non_implemented_order(order: int) -> None:
     with pytest.raises(ValueError):
-        gpflow.kernels.ArcCosine(order=order)
+        ArcCosine(order=order)
 
 
 @pytest.mark.parametrize("D, N", [[1, 4]])
-def test_arccosine_nan_gradient(D, N):
+def test_arccosine_nan_gradient(D: int, N: int) -> None:
     X = rng.rand(N, D)
-    kernel = gpflow.kernels.ArcCosine()
+    kernel = ArcCosine()
     with tf.GradientTape() as tape:
         Kff = kernel(X)
     grads = tape.gradient(Kff, kernel.trainable_variables)
@@ -122,10 +160,10 @@ def test_arccosine_nan_gradient(D, N):
 @pytest.mark.parametrize(
     "base_class",
     [
-        gpflow.kernels.SquaredExponential,
-        gpflow.kernels.Matern12,
-        gpflow.kernels.Matern32,
-        gpflow.kernels.Matern52,
+        SquaredExponential,
+        Matern12,
+        Matern32,
+        Matern52,
     ],
 )
 @pytest.mark.parametrize(
@@ -145,11 +183,18 @@ def test_arccosine_nan_gradient(D, N):
         [5, 1.3],
     ],
 )
-def test_periodic(base_class, D, N, lengthscales, variance, period):
+def test_periodic(
+    base_class: Type[Stationary],
+    D: int,
+    N: int,
+    lengthscales: TensorType,
+    variance: TensorType,
+    period: TensorType,
+) -> None:
     X = rng.randn(N, D) if D == 1 else rng.multivariate_normal(np.zeros(D), np.eye(D), N)
 
     base_kernel = base_class(lengthscales=lengthscales, variance=variance)
-    kernel = gpflow.kernels.Periodic(base_kernel, period=period)
+    kernel = Periodic(base_kernel, period=period)
     gram_matrix = kernel(X)
     reference_gram_matrix = ref_periodic_kernel(
         X, base_class.__name__, lengthscales, variance, period
@@ -161,57 +206,57 @@ def test_periodic(base_class, D, N, lengthscales, variance, period):
 @pytest.mark.parametrize(
     "base_class",
     [
-        gpflow.kernels.SquaredExponential,
-        gpflow.kernels.Matern12,
+        SquaredExponential,
+        Matern12,
     ],
 )
-def test_periodic_diag(base_class):
+def test_periodic_diag(base_class: Type[Stationary]) -> None:
     N, D = 5, 3
     X = rng.multivariate_normal(np.zeros(D), np.eye(D), N)
     base_kernel = base_class(lengthscales=2.0, variance=1.0)
-    kernel = gpflow.kernels.Periodic(base_kernel, period=6.0)
+    kernel = Periodic(base_kernel, period=6.0)
     assert_allclose(base_kernel(X, full_cov=False), kernel(X, full_cov=False))
 
 
-def test_periodic_non_stationary_base_kernel():
+def test_periodic_non_stationary_base_kernel() -> None:
     error_msg = r"Periodic requires an IsotropicStationary kernel as the `base_kernel`"
     with pytest.raises(TypeError, match=error_msg):
-        gpflow.kernels.Periodic(gpflow.kernels.Linear())
+        Periodic(Linear())
 
 
-def test_periodic_bad_ard_period():
+def test_periodic_bad_ard_period() -> None:
     error_msg = r"Size of `active_dims` \[1 2\] does not match size of ard parameter \(3\)"
-    base_kernel = gpflow.kernels.RBF(active_dims=[1, 2])
+    base_kernel = RBF(active_dims=[1, 2])
     with pytest.raises(ValueError, match=error_msg):
-        gpflow.kernels.Periodic(base_kernel, period=[1.0, 1.0, 1.0])
+        Periodic(base_kernel, period=[1.0, 1.0, 1.0])
 
 
-kernel_setups = [
+kernel_setups: Tuple[Kernel, ...] = tuple(
     kernel()
-    for kernel in gpflow.ci_utils.subclasses(gpflow.kernels.Stationary)
-    if kernel not in (gpflow.kernels.IsotropicStationary, gpflow.kernels.AnisotropicStationary)
-] + [
-    gpflow.kernels.Constant(),
-    gpflow.kernels.Linear(),
-    gpflow.kernels.Polynomial(),
-    gpflow.kernels.ArcCosine(),
-]
+    for kernel in gpflow.ci_utils.subclasses(Stationary)
+    if kernel not in (IsotropicStationary, AnisotropicStationary)
+) + (
+    Constant(),
+    Linear(),
+    Polynomial(),
+    ArcCosine(),
+)
 
 
 @pytest.mark.parametrize("D", [1, 5])
 @pytest.mark.parametrize("kernel", kernel_setups)
 @pytest.mark.parametrize("N", [10])
-def test_kernel_symmetry_1d_and_5d(D, kernel, N):
+def test_kernel_symmetry_1d_and_5d(D: int, kernel: Kernel, N: int) -> None:
     X = rng.randn(N, D)
     errors = kernel(X) - kernel(X, X)
     assert np.allclose(errors, 0)
 
 
 @pytest.mark.parametrize("N, N2, input_dim, output_dim, rank", [[10, 12, 1, 3, 2]])
-def test_coregion_shape(N, N2, input_dim, output_dim, rank):
+def test_coregion_shape(N: int, N2: int, input_dim: int, output_dim: int, rank: int) -> None:
     X = np.random.randint(0, output_dim, (N, input_dim))
     X2 = np.random.randint(0, output_dim, (N2, input_dim))
-    kernel = gpflow.kernels.Coregion(output_dim=output_dim, rank=rank)
+    kernel = Coregion(output_dim=output_dim, rank=rank)
     kernel.W = rng.randn(output_dim, rank)
     kernel.kappa = rng.randn(output_dim, 1).reshape(-1) + 1.0
 
@@ -222,9 +267,9 @@ def test_coregion_shape(N, N2, input_dim, output_dim, rank):
 
 
 @pytest.mark.parametrize("N, input_dim, output_dim, rank", [[10, 1, 3, 2]])
-def test_coregion_diag(N, input_dim, output_dim, rank):
+def test_coregion_diag(N: int, input_dim: int, output_dim: int, rank: int) -> None:
     X = np.random.randint(0, output_dim, (N, input_dim))
-    kernel = gpflow.kernels.Coregion(output_dim=output_dim, rank=rank)
+    kernel = Coregion(output_dim=output_dim, rank=rank)
     kernel.W = rng.randn(output_dim, rank)
     kernel.kappa = rng.randn(output_dim, 1).reshape(-1) + 1.0
 
@@ -234,13 +279,13 @@ def test_coregion_diag(N, input_dim, output_dim, rank):
 
 
 @pytest.mark.parametrize("N, input_dim, output_dim, rank", [[10, 1, 3, 2]])
-def test_coregion_slice(N, input_dim, output_dim, rank):
+def test_coregion_slice(N: int, input_dim: int, output_dim: int, rank: int) -> None:
     X = np.random.randint(0, output_dim, (N, input_dim))
     X = np.hstack((X, rng.randn(10, 1)))
-    kernel1 = gpflow.kernels.Coregion(output_dim=output_dim, rank=rank, active_dims=[0])
+    kernel1 = Coregion(output_dim=output_dim, rank=rank, active_dims=[0])
     # compute another kernel with additinoal inputs,
     # make sure out kernel is still okay.
-    kernel2 = gpflow.kernels.SquaredExponential(active_dims=[1])
+    kernel2 = SquaredExponential(active_dims=[1])
     kernel_prod = kernel1 * kernel2
     K1 = kernel_prod(X)
     K2 = kernel1(X) * kernel2(X)  # slicing happens inside kernel
@@ -248,28 +293,28 @@ def test_coregion_slice(N, input_dim, output_dim, rank):
 
 
 _dim = 3
-kernel_setups_extended = (
+kernel_setups_extended: Tuple[Kernel, ...] = (
     kernel_setups
-    + [
+    + (
         SquaredExponential() + Linear(),
         SquaredExponential() * Linear(),
         SquaredExponential() + Linear(variance=rng.rand(_dim)),
-    ]
-    + [ArcCosine(order=order) for order in ArcCosine.implemented_orders]
+    )
+    + tuple(ArcCosine(order=order) for order in ArcCosine.implemented_orders)
 )
 
 
 @pytest.mark.parametrize("kernel", kernel_setups_extended)
 @pytest.mark.parametrize("N, dim", [[30, _dim]])
-def test_diags(kernel, N, dim):
+def test_diags(kernel: Kernel, N: int, dim: int) -> None:
     X = np.random.randn(N, dim)
     kernel1 = tf.linalg.diag_part(kernel(X, full_cov=True))
     kernel2 = kernel(X, full_cov=False)
     assert np.allclose(kernel1, kernel2)
 
 
-def test_conv_diag():
-    kernel = gpflow.kernels.Convolutional(gpflow.kernels.SquaredExponential(), [3, 3], [2, 2])
+def test_conv_diag() -> None:
+    kernel = Convolutional(SquaredExponential(), [3, 3], [2, 2])
     X = np.random.randn(3, 9)
     kernel_full = np.diagonal(kernel(X, full_cov=True))
     kernel_diag = kernel(X, full_cov=False)
@@ -278,15 +323,15 @@ def test_conv_diag():
 
 # Add a rbf and linear kernel, make sure the result is the same as adding the result of
 # the kernels separately.
-_kernel_setups_add = [
-    gpflow.kernels.SquaredExponential(),
-    gpflow.kernels.Linear(),
-    gpflow.kernels.SquaredExponential() + gpflow.kernels.Linear(),
-]
+_kernel_setups_add: Tuple[Kernel, ...] = (
+    SquaredExponential(),
+    Linear(),
+    SquaredExponential() + Linear(),
+)
 
 
 @pytest.mark.parametrize("N, D", [[10, 1]])
-def test_add_symmetric(N, D):
+def test_add_symmetric(N: int, D: int) -> None:
     X = rng.randn(N, D)
     Kffs = [kernel(X) for kernel in _kernel_setups_add]
 
@@ -294,7 +339,7 @@ def test_add_symmetric(N, D):
 
 
 @pytest.mark.parametrize("N, M, D", [[10, 12, 1]])
-def test_add_asymmetric(N, M, D):
+def test_add_asymmetric(N: int, M: int, D: int) -> None:
     X, Z = rng.randn(N, D), rng.randn(M, D)
     Kfus = [kernel(X, Z) for kernel in _kernel_setups_add]
 
@@ -302,13 +347,13 @@ def test_add_asymmetric(N, M, D):
 
 
 @pytest.mark.parametrize("N, D", [[10, 1]])
-def test_white(N, D):
+def test_white(N: int, D: int) -> None:
     """
     The white kernel should not give the same result when called with k(X) and
     k(X, X)
     """
     X = rng.randn(N, D)
-    kernel = gpflow.kernels.White()
+    kernel = White()
     Kff_sym = kernel(X)
     Kff_asym = kernel(X, X)
 
@@ -317,12 +362,12 @@ def test_white(N, D):
 
 _kernel_classes_slice = [
     kernel
-    for kernel in gpflow.ci_utils.subclasses(gpflow.kernels.Stationary)
-    if kernel not in (gpflow.kernels.IsotropicStationary, gpflow.kernels.AnisotropicStationary)
+    for kernel in gpflow.ci_utils.subclasses(Stationary)
+    if kernel not in (IsotropicStationary, AnisotropicStationary)
 ] + [
-    gpflow.kernels.Constant,
-    gpflow.kernels.Linear,
-    gpflow.kernels.Polynomial,
+    Constant,
+    Linear,
+    Polynomial,
 ]
 
 _kernel_triples_slice = [
@@ -333,7 +378,7 @@ _kernel_triples_slice = [
 
 @pytest.mark.parametrize("kernel_triple", _kernel_triples_slice)
 @pytest.mark.parametrize("N, D", [[20, 2]])
-def test_slice_symmetric(kernel_triple, N, D):
+def test_slice_symmetric(kernel_triple: Tuple[Kernel, Kernel, Kernel], N: int, D: int) -> None:
     X = rng.randn(N, D)
     K1, K3 = kernel_triple[0](X), kernel_triple[2](X[:, :1])
     assert np.allclose(K1, K3)
@@ -343,7 +388,9 @@ def test_slice_symmetric(kernel_triple, N, D):
 
 @pytest.mark.parametrize("kernel_triple", _kernel_triples_slice)
 @pytest.mark.parametrize("N, M, D", [[10, 12, 2]])
-def test_slice_asymmetric(kernel_triple, N, M, D):
+def test_slice_asymmetric(
+    kernel_triple: Tuple[Kernel, Kernel, Kernel], N: int, M: int, D: int
+) -> None:
     X = rng.randn(N, D)
     Z = rng.randn(M, D)
     K1, K3 = kernel_triple[0](X, Z), kernel_triple[2](X[:, :1], Z[:, :1])
@@ -352,15 +399,15 @@ def test_slice_asymmetric(kernel_triple, N, M, D):
     assert np.allclose(K2, K4)
 
 
-_kernel_setups_prod = [
-    gpflow.kernels.Matern32(),
-    gpflow.kernels.Matern52(lengthscales=0.3),
-    gpflow.kernels.Matern32() * gpflow.kernels.Matern52(lengthscales=0.3),
-]
+_kernel_setups_prod: Tuple[Kernel, ...] = (
+    Matern32(),
+    Matern52(lengthscales=0.3),
+    Matern32() * Matern52(lengthscales=0.3),
+)
 
 
 @pytest.mark.parametrize("N, D", [[30, 2]])
-def test_product(N, D):
+def test_product(N: int, D: int) -> None:
     X = rng.randn(N, D)
     Kffs = [kernel(X) for kernel in _kernel_setups_prod]
 
@@ -368,7 +415,7 @@ def test_product(N, D):
 
 
 @pytest.mark.parametrize("N, D", [[30, 4], [10, 7]])
-def test_active_product(N, D):
+def test_active_product(N: int, D: int) -> None:
     X = rng.randn(N, D)
     dims, rand_idx, ls = (
         list(range(D)),
@@ -382,7 +429,7 @@ def test_active_product(N, D):
         ls,
     ]
     kernels = [
-        gpflow.kernels.SquaredExponential(lengthscales=lengthscales, active_dims=dims)
+        SquaredExponential(lengthscales=lengthscales, active_dims=dims)
         for dims, lengthscales in zip(active_dims_list, lengthscales_list)
     ]
     kernel_prod = kernels[0] * kernels[1]
@@ -394,30 +441,30 @@ def test_active_product(N, D):
 
 
 @pytest.mark.parametrize("D", [4, 7])
-def test_ard_init_scalar(D):
+def test_ard_init_scalar(D: int) -> None:
     """
     For ard kernels, make sure that kernels can be instantiated with a single
     scalar lengthscale or a suitable array of lengthscales
     """
-    kernel_1 = gpflow.kernels.SquaredExponential(lengthscales=2.3)
-    kernel_2 = gpflow.kernels.SquaredExponential(lengthscales=np.ones(D) * 2.3)
+    kernel_1 = SquaredExponential(lengthscales=2.3)
+    kernel_2 = SquaredExponential(lengthscales=np.ones(D) * 2.3)
     lengthscales_1 = kernel_1.lengthscales.numpy()
     lengthscales_2 = kernel_2.lengthscales.numpy()
     assert np.allclose(lengthscales_1, lengthscales_2, atol=1e-10)
 
 
-def test_ard_invalid_active_dims():
+def test_ard_invalid_active_dims() -> None:
     msg = r"Size of `active_dims` \[1\] does not match size of ard parameter \(2\)"
     with pytest.raises(ValueError, match=msg):
-        gpflow.kernels.SquaredExponential(lengthscales=np.ones(2), active_dims=[1])
+        SquaredExponential(lengthscales=np.ones(2), active_dims=[1])
 
 
 @pytest.mark.parametrize(
     "kernel_class, param_name",
     [
-        [gpflow.kernels.SquaredExponential, "lengthscales"],
-        [gpflow.kernels.Linear, "variance"],
-        [gpflow.kernels.ArcCosine, "weight_variances"],
+        [SquaredExponential, "lengthscales"],
+        [Linear, "variance"],
+        [ArcCosine, "weight_variances"],
     ],
 )
 @pytest.mark.parametrize(
@@ -428,7 +475,9 @@ def test_ard_invalid_active_dims():
         [[1.0, 1.0], True],
     ],
 )
-def test_ard_property(kernel_class, param_name, param_value, ard):
+def test_ard_property(
+    kernel_class: Type[Kernel], param_name: str, param_value: Any, ard: bool
+) -> None:
     kernel = kernel_class(**{param_name: param_value})
     assert kernel.ard is ard
 
@@ -450,18 +499,25 @@ def test_ard_property(kernel_class, param_name, param_value, ard):
         ],
     ],
 )
-def test_changepoints_init_fail(locations, steepness, error_msg):
-    kernels = [
-        gpflow.kernels.Matern12(),
-        gpflow.kernels.Linear(),
-        gpflow.kernels.Matern32(),
+def test_changepoints_init_fail(
+    locations: Sequence[float], steepness: Union[float, Sequence[float]], error_msg: str
+) -> None:
+    kernels: Sequence[Kernel] = [
+        Matern12(),
+        Linear(),
+        Matern32(),
     ]
     with pytest.raises(ValueError, match=error_msg):
-        gpflow.kernels.ChangePoints(kernels, locations, steepness)
+        ChangePoints(kernels, locations, steepness)
 
 
-def _assert_changepoints_kern_err(X, kernels, locations, steepness):
-    kernel = gpflow.kernels.ChangePoints(kernels, locations, steepness=steepness)
+def _assert_changepoints_kern_err(
+    X: TensorType,
+    kernels: Sequence[Kernel],
+    locations: Sequence[float],
+    steepness: Union[float, Sequence[float]],
+) -> None:
+    kernel = ChangePoints(kernels, locations, steepness=steepness)
     reference_gram_matrix = _ref_changepoints(X, kernels, locations, steepness)
 
     assert_allclose(kernel(X), reference_gram_matrix)
@@ -473,13 +529,13 @@ def _assert_changepoints_kern_err(X, kernels, locations, steepness):
     "kernels, locations, steepness",
     [
         # 1. Single changepoint
-        [[gpflow.kernels.Constant(), gpflow.kernels.Constant()], [2.0], 5.0],
+        [[Constant(), Constant()], [2.0], 5.0],
         # 2. Two changepoints
         [
             [
-                gpflow.kernels.Constant(),
-                gpflow.kernels.Constant(),
-                gpflow.kernels.Constant(),
+                Constant(),
+                Constant(),
+                Constant(),
             ],
             [1.0, 2.0],
             5.0,
@@ -487,9 +543,9 @@ def _assert_changepoints_kern_err(X, kernels, locations, steepness):
         # 3. Multiple steepness
         [
             [
-                gpflow.kernels.Constant(),
-                gpflow.kernels.Constant(),
-                gpflow.kernels.Constant(),
+                Constant(),
+                Constant(),
+                Constant(),
             ],
             [1.0, 2.0],
             [5.0, 10.0],
@@ -497,17 +553,22 @@ def _assert_changepoints_kern_err(X, kernels, locations, steepness):
         # 4. Variety of kernels
         [
             [
-                gpflow.kernels.Matern12(),
-                gpflow.kernels.Linear(),
-                gpflow.kernels.SquaredExponential(),
-                gpflow.kernels.Constant(),
+                Matern12(),
+                Linear(),
+                SquaredExponential(),
+                Constant(),
             ],
             [1.0, 2.0, 3.0],
             5.0,
         ],
     ],
 )
-def test_changepoints(N, kernels, locations, steepness):
+def test_changepoints(
+    N: int,
+    kernels: Sequence[Kernel],
+    locations: Sequence[float],
+    steepness: Union[float, Sequence[float]],
+) -> None:
     X_data = rng.randn(N, 1)
     _assert_changepoints_kern_err(X_data, kernels, locations, steepness)
 
@@ -522,9 +583,13 @@ def test_changepoints(N, kernels, locations, steepness):
         [[1, 2, 3], [4, 5, 6], True],
     ],
 )
-def test_on_separate_dims(active_dims_1, active_dims_2, is_separate):
-    kernel_1 = gpflow.kernels.Linear(active_dims=active_dims_1)
-    kernel_2 = gpflow.kernels.SquaredExponential(active_dims=active_dims_2)
+def test_on_separate_dims(
+    active_dims_1: Optional[Sequence[int]],
+    active_dims_2: Optional[Sequence[int]],
+    is_separate: bool,
+) -> None:
+    kernel_1 = Linear(active_dims=active_dims_1)
+    kernel_2 = SquaredExponential(active_dims=active_dims_2)
     assert kernel_1.on_separate_dims(kernel_2) == is_separate
     assert kernel_2.on_separate_dims(kernel_1) == is_separate
     assert kernel_1.on_separate_dims(kernel_1) is False
@@ -532,7 +597,7 @@ def test_on_separate_dims(active_dims_1, active_dims_2, is_separate):
 
 
 @pytest.mark.parametrize("kernel", kernel_setups_extended)
-def test_kernel_call_diag_and_X2_errors(kernel):
+def test_kernel_call_diag_and_X2_errors(kernel: Kernel) -> None:
     X = rng.randn(4, 1)
     X2 = rng.randn(5, 1)
 
@@ -540,10 +605,10 @@ def test_kernel_call_diag_and_X2_errors(kernel):
         kernel(X, X2, full_cov=False)
 
 
-def test_periodic_active_dims_matches():
+def test_periodic_active_dims_matches() -> None:
     active_dims = [1]
-    base_kernel = gpflow.kernels.SquaredExponential(active_dims=active_dims)
-    kernel = gpflow.kernels.Periodic(base_kernel=base_kernel)
+    base_kernel = SquaredExponential(active_dims=active_dims)
+    kernel = Periodic(base_kernel=base_kernel)
 
     assert kernel.active_dims == base_kernel.active_dims
 
@@ -554,24 +619,24 @@ def test_periodic_active_dims_matches():
     assert kernel.active_dims == base_kernel.active_dims
 
 
-def test_latent_kernels():
-    kernel_list = [SquaredExponential(), White(), White() + Linear()]
+def test_latent_kernels() -> None:
+    kernel_list: Tuple[Kernel, ...] = (SquaredExponential(), White(), White() + Linear())
 
-    multioutput_kernel_list = [
+    multioutput_kernel_list: Tuple[MultioutputKernel, ...] = (
         SharedIndependent(SquaredExponential(), 3),
         SeparateIndependent(kernel_list),
         LinearCoregionalization(kernel_list, np.random.random((5, 3))),
-    ]
+    )
     assert len(multioutput_kernel_list[0].latent_kernels) == 1
     assert multioutput_kernel_list[1].latent_kernels == tuple(kernel_list)
     assert multioutput_kernel_list[2].latent_kernels == tuple(kernel_list)
 
 
-def test_combination_LMC_kernels():
+def test_combination_LMC_kernels() -> None:
     N, D, P = 100, 3, 2
-    kernel_list1 = [Linear(active_dims=[1]), SquaredExponential()]
+    kernel_list1: Tuple[Kernel, ...] = (Linear(active_dims=[1]), SquaredExponential())
     L1 = len(kernel_list1)
-    kernel_list2 = [SquaredExponential(), Linear(), Linear()]
+    kernel_list2: Tuple[Kernel, ...] = (SquaredExponential(), Linear(), Linear())
     L2 = len(kernel_list2)
     k1 = LinearCoregionalization(kernel_list1, np.random.randn(P, L1))
     k2 = LinearCoregionalization(kernel_list2, np.random.randn(P, L2))
