@@ -6,7 +6,7 @@ import gpflow
 @pytest.mark.parametrize(
     "init_lower", [0.0, 1e-6, gpflow.likelihoods.Gaussian.DEFAULT_VARIANCE_LOWER_BOUND]
 )
-def test_gaussian_lower_bound_constructor_check(init_lower):
+def test_gaussian_lower_bound_constructor_check(init_lower: float) -> None:
     with pytest.raises(
         ValueError, match="variance of the Gaussian likelihood must be strictly greater than"
     ):
