@@ -12,11 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Optional
-
 import numpy as np
 import pytest
-import tensorflow as tf
 
 import gpflow
 from gpflow.utilities.ops import pca_reduce
@@ -58,7 +55,7 @@ def test_bayesian_gplvm_1d():
         kernel,
         inducing_variable=inducing_variable,
     )
-    assert len(m.inducing_variable) == Data.M
+    assert m.inducing_variable.num_inducing == Data.M
 
     elbo_initial = m.elbo()
     opt = gpflow.optimizers.Scipy()
