@@ -17,11 +17,10 @@ from dataclasses import dataclass
 import numpy as np
 import pytest
 import tensorflow as tf
-from numpy.testing import assert_allclose, assert_array_equal, assert_array_less
+from numpy.testing import assert_allclose
 
 import gpflow
 from gpflow import set_trainable
-from gpflow.config import default_float
 
 
 @dataclass(frozen=True)
